#include <stdio.h>
#include <string.h>
#include <xmms/plugin.h>
#include <xmms/util.h>
#include <netmd.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

NetMD *md = 0;

static void xmms_netmd_init(void);
static void xmms_netmd_about(void);
static void xmms_netmd_configure(void);
static int xmms_netmd_is_our_file(char *filename);
static GList *xmms_netmd_scan_dir(char *dirname);
static void xmms_netmd_play_file(char *filename);
static void xmms_netmd_stop(void);
static void xmms_netmd_pause(short paused);
static void xmms_netmd_seek(int time);
static int xmms_netmd_get_time(void);
//static void xmms_netmd_get_volume(int *l, int *r);
//static void xmms_netmd_set_volume(int l, int r);
static void xmms_netmd_cleanup(void);
static void xmms_netmd_get_song_info(char *filename, char **title, int *length);
static void xmms_netmd_file_info_box(char *filename);

InputPlugin xmms_netmd = {
  0, 0, "NetMD Plugin", xmms_netmd_init, xmms_netmd_about,
  0 /*xmms_netmd_configure*/, xmms_netmd_is_our_file, xmms_netmd_scan_dir,
  xmms_netmd_play_file, xmms_netmd_stop, xmms_netmd_pause, xmms_netmd_seek,
  0 /*set_eq*/, xmms_netmd_get_time, 0 /*xmms_netmd_get_volume*/,
  0 /*xmms_netmd_set_volume*/, xmms_netmd_cleanup, 0, 0, 0, 0,
  xmms_netmd_get_song_info, xmms_netmd_file_info_box, 0
};

// XMMS entry point
InputPlugin *get_iplugin_info(void) {
  return &xmms_netmd;
}

static void xmms_netmd_init(void) {
  //printf("xmms_netmd_init called\n");
  g_type_init();
  md = NETMD(netmd_new());
  if (!md)
    fprintf(stderr, "Hmm.. netmd_new() returned null\n");
  netmd_get_disc(md);
}

static void xmms_netmd_about(void) {
  //printf("xmms_netmd_about called\n");
  static GtkWidget *box;
  box = xmms_show_message(
    _("About XMMS-NetMD"),
    _("XMMS NetMD Input Plugin by Pete Ryland <pdr@pdr.cx>.\n\n"
    "Copyright (c) 2002-2003 Pete Ryland <pdr@pdr.cx>\n\n"
    "Plays MDs using a NetMD device connected via USB."),
    _("Ok"), FALSE, NULL, NULL);
  gtk_signal_connect(GTK_OBJECT(box), "destroy",
                     GTK_SIGNAL_FUNC(gtk_widget_destroyed), &box);
}

static void xmms_netmd_configure(void) {
  //printf("xmms_netmd_configure called\n");
}

int filename_to_tracknumber(char *filename) {
  char *cp;
  unsigned long track_number;
  //printf("filename_to_tracknumber(\"%s\") called\n", filename);
  if (strncmp(filename, "netmd:///", 9) != 0)
    return -1;
  track_number = strtoul(filename+9, &cp, 16);
  if (*cp != '\0')
    return -1;
  //printf("track number is %d\n", (int)track_number);
  return track_number;
}

static int xmms_netmd_is_our_file(char *filename) {
  int track_number;
  //printf("xmms_netmd_is_our_file(\"%s\") called\n", filename);
  track_number = filename_to_tracknumber(filename);
  return (track_number > 0 && track_number <= md->disc->numtracks)? TRUE : FALSE;
}

static GList *xmms_netmd_scan_dir(char *dirname) {
  NetMDDisc *disc;
  GList *r;
  char *cp;
  unsigned i;
  char buff[16];

  //printf("xmms_netmd_scan_dir(\"%s\") called\n", dirname);

  if (!md) {
    fprintf(stderr, "xmms_netmd: ERROR! md null in xmms_netmd_scan_dir!");
    return 0;
  }

  disc = netmd_get_disc(md);
  if (strcmp(dirname, "netmd:///") != 0)
    return 0;

  r = g_list_alloc();
  for (i = 1; i <= disc->numtracks; i++) {
    snprintf(buff, 16, "%02x", i);
    g_list_append(r, strdup(buff));
    //printf("appending %s\n", buff);
  }

  return r;
}

unsigned long track_number;

static void xmms_netmd_play_file(char *filename) {
  char *cp;
  NetMDTrack *track;
  //printf("xmms_netmd_play_file called\n");
  track_number = (int)filename_to_tracknumber(filename);
  //printf("track number is %d\n", (int)track_number);
  if (!md || !md->disc)
    fprintf(stderr, "netmd problem\n");
  if (track_number <= 0 || track_number > md->disc->numtracks)
    return;
  track = md->disc->tracks[track_number-1];
  netmd_playback_set_track(md, (int)track_number);
  netmd_playback_start(md);
  //printf("setting track info\n");
  xmms_netmd.set_info(track->title, ((track->mins * 60 + track->secs) * 100 + track->hunds) * 10, 100000, 100000 /*get from codec*/, 0);
}

static void xmms_netmd_stop(void) {
  //printf("xmms_netmd_stop called\n");
  netmd_playback_stop(md);
}

static void xmms_netmd_pause(short paused) {
  //printf("xmms_netmd_pause(%d) called\n", paused);
  if (md) {
    if (paused)
      netmd_playback_pause(md);
    else
      netmd_playback_start(md);
  }
}

int seeking_time = 0;

static void xmms_netmd_seek(int time) {
  //printf("xmms_netmd_seek called(%d)\n", time);
  netmd_playback_set_position(md, time*100L);
  seeking_time = time;
}

static int xmms_netmd_get_time(void) {
  static int i = -1;
  static int lastval = 0;

  // terrible hack coz it polls more frequently than netmd can respond, and
  // hence introduces nasty race conditions.
  i++; i%=10;
  if (i) {
    return lastval;
  }

  if (netmd_playback_get_track(md) != track_number) {
    netmd_playback_pause(md);
    i = -1;
    seeking_time = lastval = 0;
    return -1;
  }

  //printf("xmms_netmd_get_time called\n");
  if (seeking_time) {
    long pp = netmd_playback_get_position(md);
    long diff = labs(pp-seeking_time*100);
    //printf("seeking_time is %ld\n", seeking_time);
    //printf("diff is %ld\n", diff);
    if (diff > 200) {
      return lastval = seeking_time*1000;
    } else {
      seeking_time = 0;
      return lastval = 10*pp;
    }
  }
  int r = 10*netmd_playback_get_position(md);
  //printf("returning %2d\n", r);
  return lastval = r;
}

/*
static void xmms_netmd_get_volume(int *l, int *r) {
  //printf("xmms_netmd_get_volume called\n");
  // *l = *r = 50;
}

static void xmms_netmd_set_volume(int l, int r) {
  //printf("xmms_netmd_set_volume called\n");
}
*/

static void xmms_netmd_cleanup(void) {
  //printf("xmms_netmd_cleanup called\n");
}

static void xmms_netmd_get_song_info(char *filename, char **title, int *length) {
  int track_number;
  NetMDTrack *track;
  //printf("xmms_netmd_get_song_info(\"%s\") called\n", filename);
  track_number = (int)filename_to_tracknumber(filename);
  if (track_number <= 0 || track_number > md->disc->numtracks)
    return;
  track = md->disc->tracks[track_number-1];
  *title = strdup(track->title);
  *length = ((track->mins * 60 + track->secs) * 100 + track->hunds) * 10;
}

static void xmms_netmd_file_info_box(char *filename) {
  //printf("xmms_netmd_file_info_box called\n");
}
